﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace VarePortableClassLibrary
{
    // I eksemplet er der nu sat set-metoder på alle properties da de ellers ikke kan serialiseres

    // definition af Vare (data og metoder der findes for hver enkelt vare-object
    [DataContract]
    public class Vare : IVare
    {

        private int varenr;
        [DataMember]
        public int Varenr               // property - fra interface kan der kun aflæses
        {
            get { return varenr; }
            protected set { varenr = value; }
        }

        private string betegnelse;
        [DataMember]
        public string Betegnelse        // property - kan både aflæeses og ændres
        {
            get { return betegnelse; }
            set { betegnelse = value; }
        }

        private int antalPåLager;
        [DataMember]
        public int AntalPåLager         // property - fra interface kan der kun aflæses
        // ændringer kan ske med metoden for
        {                               // registrering af antal købte og solgte
            get { return antalPåLager; }
            set { antalPåLager = value; }
        }

        private float salgsEnhedsPris;			// aktuel excl. moms
        [DataMember]
        public float SalgsEnhedsPris    // property - kan både aflæeses og ændres
        {
            get { return salgsEnhedsPris; }
            set { salgsEnhedsPris = value; }
        }

        private float indkøbsEnhedsPris;			// aktuel excl. moms
        [DataMember]
        public float IndkøbsEnhedsPris  // property - kan både aflæeses og ændres
        {
            get { return indkøbsEnhedsPris; }
            set { indkøbsEnhedsPris = value; }
        }

        private string enhed;
        [DataMember]
        public string Enhed             // property - kan både aflæeses og ændres
        {
            get { return enhed; }
            set { enhed = value; }
        }
        private int solgtAntal;
        [DataMember]
        public int SolgtAntal           // property - kan kun aflæses direkte
        // ændringer kan ske med metoden for
        {                               // registrering af antal solgte
            get { return solgtAntal; }
            set { solgtAntal = value; }
        }
        private int indkøbtAntal;
        [DataMember]
        public int IndkøbtAntal         // property - fra interface kan der kun aflæses
        // ændringer kan ske med metoden for
        {                               // registrering af antal købte
            get { return indkøbtAntal; }
            set { indkøbsEnhedsPris = value; }
        }

        private int minimumAntal;
        [DataMember]
        public int MinimumAntal         // property - kan både aflæeses og ændres
        {
            get { return minimumAntal; }
            set { minimumAntal = value; }
        }

        // constructor til brug for skabelse af nyt vare-object
        public Vare(int varenr, string betegnelse, int antalPåLager,
                    float salgsEnhedsPris, string enhed, float indkøbsEnhedsPris,
                    int solgtAntal, int indkøbtAntal, int minimumAntal)
        {
            this.varenr = varenr;
            this.betegnelse = betegnelse;
            this.antalPåLager = antalPåLager;
            this.salgsEnhedsPris = salgsEnhedsPris;
            this.enhed = enhed;
            this.indkøbsEnhedsPris = indkøbsEnhedsPris;
            this.solgtAntal = solgtAntal;
            this.indkøbtAntal = indkøbtAntal;
            this.minimumAntal = minimumAntal;
        }

        // metode for beregning af den salgsværdien det samlede antal
        public double Salgsværdi()
        {
            double salgsværdi = ((double)this.antalPåLager) * this.salgsEnhedsPris;
            return salgsværdi;
        }

        // metode for beregning af den indkøbsværdien det samlede antal
        public double Indkøbsværdi()
        {
            double indkøbsværdi = checked(this.AntalPåLager * this.IndkøbsEnhedsPris);
            return indkøbsværdi;
        }

        // metode for registreing af et antal solgte enheder af dette vareobjekt
        public void RegistrerVarerSolgt(int antal)
        {
            this.solgtAntal = this.solgtAntal + antal;
            this.antalPåLager = this.antalPåLager - antal;
        }

        // metode for registreing af et antal indkøbte enheder af dette vareobjekt
        public void RegistrerVarerIndkøbt(int antal)
        {
            this.indkøbtAntal = this.indkøbtAntal + antal;
            this.antalPåLager = this.antalPåLager + antal;
        }

        public override string ToString()
        {
            return ""+betegnelse;
        }
    }
}
